# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007-2009 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+space".
"""

shortcut = 'ctrl+ '
sticky = False

class WordComplete(object):
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
    
    def activate(self):
        """
        Activate word complete.
        """

        view = self.document_manager.get_active_view()
        doc = self.document_manager.get_active_view().buffer
        
        insert_mark = doc.get_insert()
        insert_iter = doc.get_iter_at_mark(insert_mark)
        start_iter = insert_iter.copy()

        self.__find_word_start(start_iter)
        key = doc.get_text(start_iter, insert_iter)
        
        if key != "":
            if view.word_complete.pw == None:
                view.word_complete.complete(key)
            return True
        else:
            return False
            
    
#    def __iter_after_underline(self, iterator):
#        """
#        Verify if iterator is after a underline character.
#        
#        @param iterator: the iterator verified
#        @type iterator: a TextIter.
#        """
#        
#        if iterator.backward_char():
#            char = iterator.get_char()
#            if char == "_":
#                iterator.forward_char()
#                return True
#            else:
#                iterator.forward_char()
#                return False
#        else:
#            return False
    
    def __find_word_start(self, iterator):
        """
        Move iterator to word start ignoring underlines.
        
        @param iterator: the iterator moved.
        @type iterator: a TextIter.
        """
        
        while True:
            if iterator.backward_word_start():
                if iterator.backward_char():
                    char = iterator.get_char()
                    if char == "_":
                        continue
                    else:
                        iterator.forward_char()
                        return True
                else:
                    return True
            else:
                return False

